import logging
from aiogram import Bot, types
from aiogram.dispatcher import Dispatcher, FSMContext
from aiogram.dispatcher.filters import Text
from aiogram.dispatcher.filters.state import State, StatesGroup
from aiogram.utils import executor
from aiogram.types import KeyboardButton, ReplyKeyboardRemove, ReplyKeyboardMarkup
from aiogram.contrib.fsm_storage.memory import MemoryStorage
import psycopg2
from config import password, host, user, db_name,  token

"""**************************************** service PART *******************************"""
bot = Bot(token=token)
memory = MemoryStorage()
dp = Dispatcher(bot, storage=memory)
logging.basicConfig(level=logging.INFO)

but1 = KeyboardButton("/load")
but2 = KeyboardButton("/show")
but3 = KeyboardButton("Фото")
but4 = KeyboardButton("Видео")
but5 = KeyboardButton("Документ")
keyboard_start = ReplyKeyboardMarkup(resize_keyboard=True, one_time_keyboard=True)
keyboard_start.row(but1, but2)
keyboard_type = ReplyKeyboardMarkup(resize_keyboard=True, one_time_keyboard=True)
keyboard_type.row(but3, but4, but5)
keyboard_list = ReplyKeyboardMarkup(resize_keyboard=True, one_time_keyboard=True)
keyboard_list.add(KeyboardButton("Показать список"))

class FSMload(StatesGroup):
    load_type = State()
    show_type = State()
    photo = State()
    text = State()
    video = State()
    showphoto = State()
    showtext = State()
    showvideo = State()
    load_name = State()
    show_name = State()

"""****************************************SQL PART*******************************"""

try:
    connect = psycopg2.connect(
        host = host,
        user = user,
        password = password,
        database = db_name
    )
    connect.autocommit = True
    with connect.cursor() as cursor:
        cursor.execute("""
        CREATE TABLE IF NOT EXISTS data
        (
            name CHARACTER VARYING(50) PRIMARY KEY,
            id CHARACTER VARYING(100) NOT NULL,
            type CHARACTER VARYING(10) NOT NULL
        )
        """)
except Exception as _ex:
    print("Something wrong with database\nError: ", _ex)

"""****************************************COMMON PART*******************************"""


@dp.message_handler(commands=["start", "help"])
@dp.message_handler(Text(equals=["start", "help"],ignore_case=True))
async def start_command(message: types.Message):
    mess = message.text
    mess
    s = "Привет, начнём" if message.text in ["start", "/start","Start","/"] else "."
    await message.answer(s, reply_markup=keyboard_start)


@dp.message_handler(Text(equals=['загрузить', 'load'],ignore_case=True), state=None)
@dp.message_handler(commands=['загрузить', 'load'], state=None)
async def load_command(message: types.Message):
    await FSMload.load_name.set()
    await message.answer("введите название файла")

@dp.message_handler(state="*", commands = ['cancel','отмена'])
@dp.message_handler(Text(equals=['cancel','отмена'],ignore_case = True), state="*")
async def cancel(message: types.Message, state: FSMContext):
    current_state = await state.get_state()
    if current_state is None:
        return
    await state.finish()
    await message.reply("отменено")

@dp.message_handler(state=FSMload.load_name)
async def name_command(message: types.Message, state:FSMContext):
    with connect.cursor() as cursor:
        cursor.execute(
            f"""SELECT name
            FROM data
            where name='{message.text}'"""
        )
        if cursor.fetchone():
            await message.answer("файл с таким именем уже существует")
            await state.finish()
            return
    async with state.proxy() as my_data:
        my_data["name"] = message.text
    await FSMload.load_type.set()
    await message.answer("выберите тип файла", reply_markup=keyboard_type)

@dp.message_handler(state=FSMload.load_type)
async def load_type_command(message: types.Message, state: FSMContext):
    if message.text == "Фото":
        await message.reply("Загрузите фото", reply_markup=ReplyKeyboardRemove())
        await FSMload.photo.set()
    elif message.text == "Документ":
        await FSMload.text.set()
        await message.reply("Загрузите текст", reply_markup=ReplyKeyboardRemove())
    elif message.text == "Видео":
        await FSMload.video.set()
        await message.reply("Загрузите видео", reply_markup=ReplyKeyboardRemove())
    else:
        await message.reply("некоррекный ответ\nзапрос сброшен", reply_markup=ReplyKeyboardRemove())
        await state.finish()

@dp.message_handler(state=FSMload.photo, content_types=['photo'])
async def load_photo(message: types.Message, state: FSMContext):
    async with state.proxy() as my_data:
        my_data["data"] = message.photo[0].file_id
    with connect.cursor() as cursor:
        cursor.execute(
            f"""INSERT INTO data (name, id, type) VALUES
            ('{my_data["name"]}','{my_data["data"]}', 'photo');"""
        )
    await state.finish()


@dp.message_handler(state=FSMload.text, content_types=['document'])
async def load_text(message: types.Message, state: FSMContext):
    async with state.proxy() as my_data:
        my_data["data"] = message.document.file_id
    with connect.cursor() as cursor:
        cursor.execute(
            f"""INSERT INTO data (name, id, type) VALUES
            ('{my_data["name"]}','{my_data["data"]}', 'document');"""
        )
    await state.finish()

@dp.message_handler(state=FSMload.video, content_types=['video'])
async def load_video(message: types.Message, state: FSMContext):
    async with state.proxy() as my_data:
        my_data["data"] = message.video.file_id
    with connect.cursor() as cursor:
        cursor.execute(
            f"""INSERT INTO data (name, id, type) VALUES
            ('{my_data["name"]}','{my_data["data"]}', 'video');"""
        )
    await state.finish()

@dp.message_handler(Text(equals=["показать", "show"],ignore_case = True), state=None)
@dp.message_handler(commands=["показать", "show"], state=None)
async def show_command(message: types.Message):
    await message.answer("выберите тип файла", reply_markup=keyboard_type)
    await FSMload.show_type.set()

@dp.message_handler(state=FSMload.show_type)
async def load_type_command(message: types.Message, state: FSMContext):
    if message.text == "Фото":
        await message.reply("введите название фото или выберите из списка", reply_markup=keyboard_list)
        await FSMload.showphoto.set()
    elif message.text == "Документ":
        await FSMload.showtext.set()
        await message.reply("введите название текста или выберите из списка", reply_markup=keyboard_list)
    elif message.text == "Видео":
        await FSMload.showvideo.set()
        await message.reply("введите название видео или выберите из списка", reply_markup=keyboard_list)
    else:
        await message.reply("некоррекный ответ\nзапрос сброшен")
        await state.finish()

@dp.message_handler(state=FSMload.showphoto, content_types=['text'])
async def show_photo(message: types.Message, state: FSMContext):
    if message.text == "Показать список":
        with connect.cursor() as cursor:
            cursor.execute(
                f"""SELECT name
                FROM data
                where type = 'photo'"""
            )
            await message.answer("/"+"\n/".join([x[0] for x in cursor.fetchall()]))
            await message.answer("введите название из списка", reply_markup=ReplyKeyboardRemove())
        return
    
    name = message.text
    name = name.split("/")[-1]
    name = name.split("@")[0]
    with connect.cursor() as cursor:
        cursor.execute(
            f"""SELECT id
            FROM data
            where name = '{name}' and type = 'photo'"""
        )
        await state.finish()
        fetch = cursor.fetchone()
        if fetch:
            id = fetch[0]
            await message.answer_photo(photo=id)
        else:
            await message.answer("фото с данным именем отсутствует", reply_markup=ReplyKeyboardRemove())
            return


@dp.message_handler(state=FSMload.showtext, content_types=['text'])
async def show_text(message: types.Message, state: FSMContext):
    if message.text == "Показать список":
        with connect.cursor() as cursor:
            cursor.execute(
                f"""SELECT name
                FROM data
                where type = 'document'"""
            )
            await message.answer("/"+"\n/".join([x[0] for x in cursor.fetchall()]))
            await message.answer("введите название из списка", reply_markup=ReplyKeyboardRemove())
        return

    name = message.text
    name = name.split("/")[-1]
    name = name.split("@")[0]
    with connect.cursor() as cursor:
        cursor.execute(
            f"""SELECT id
            FROM data
            where name = '{name}' and type = 'document'"""
        )
        await state.finish()
        fetch = cursor.fetchone()
        if fetch:
            id = fetch[0]
            await message.answer_document(document=id)
        else:
            await message.answer("документ с данным именем отсутствует", reply_markup=ReplyKeyboardRemove())
            return

@dp.message_handler(state=FSMload.showvideo, content_types=['text'])
async def show_video(message: types.Message, state: FSMContext):
    if message.text == "Показать список":
        with connect.cursor() as cursor:
            cursor.execute(
                f"""SELECT name
                FROM data
                where type = 'video'"""
            )
            await message.answer("/"+"\n/".join([x[0] for x in cursor.fetchall()]))
            await message.answer("введите название из списка", reply_markup=ReplyKeyboardRemove())
        return
    
    name = message.text
    name = name.split("/")[-1]
    name = name.split("@")[0]
    with connect.cursor() as cursor:
        cursor.execute(
            f"""SELECT id
            FROM data
            where name = '{name}' and type = 'video'"""
        )
        await state.finish()
        fetch = cursor.fetchone()
        if fetch:
            id = fetch[0]
            await message.answer_video(video=id)
        else:
            await message.answer("видео с данным именем отсутствует", reply_markup=ReplyKeyboardRemove())
            return
    

async def start(self):
    print("\n\n\tбот вышел в сеть\n\n")
executor.start_polling(dp, skip_updates=True, on_startup=start)
